import { Observable, Scheduler } from 'rxjs';
import { Never } from 'javascriptutilities';
import { Type as StoreType } from './types';
export declare namespace Action {
    /**
     * Represents a dispatchable action.
     * @template T Generics parameter.
     */
    interface Type<T> {
        readonly name: string;
        readonly value: T;
    }
}
export declare namespace StateInfo {
    /**
     * Represents necessary state information.
     * @template T Generics parameter.
     */
    interface Type<State, T> {
        readonly state: State;
        readonly lastAction: Never<Action.Type<T>>;
    }
}
export declare type ActionType<T> = Action.Type<T> | T;
export declare type Reducer<State, T> = (state: State, action: Action.Type<T>) => State;
export declare type RxReducer<State, T> = (state: State) => StateInfo.Type<State, T>;
/**
 * Create reducer streams that can be used to calculate states. Beware that the
 * resulting Observable emits a function that takes a state and returns another
 * state. The value stream from the original Observable is responsible for
 * providing values to the reducer.
 * @template State Generics parameter.
 * @template T Generics parameter.
 * @param {Observable<ActionType<T>>} obs An Observable instance.
 * @param {Reducer<State, T>} reducer A Reducer instance.
 * @returns {Observable<RxReducer<State, T>>} An Observable instance.
 */
export declare function createReducer<State, T>(obs: Observable<ActionType<T>>, reducer: Reducer<State, T>): Observable<RxReducer<State, T>>;
/**
 * Create a Redux store that accepts an Array of reducer streams, then scan them
 * to derive the latest state.
 *
 * The setup should be as follows:
 *  - Define BehaviorSubject instances that accept state values.
 *  - Create reducer streams using createReducer.
 *  - Pass the resulting reducer streams to this method.
 *  - When a new state value should be updated, call next(value).
 *  - The new state will be calculated and pushed onto the store stream.
 * @template State Generics paramter.
 * @template T Generics parameter.
 * @param {State} initialState Initial state.
 * @param {Scheduler} scheduler A Scheduler instance.
 * @param {...Observable<RxReducer<State, T>>[]} reducers An Array of Observable.
 * @returns {Observable<StateInfo.Type<State, T>>} An Observable instance.
 */
export declare function create<State, T>(initialState: State, scheduler: Scheduler, ...reducers: Observable<RxReducer<State, T>>[]): Observable<StateInfo.Type<State, T>>;
/**
 * Represents a rx-based store.
 * @extends {StoreType} StoreType extension.
 * @template State Generics paramter.
 */
export interface Type<State> extends StoreType<State> {
}
/**
 * This store is optional. It only provides some convenience when dealing with
 * state streams.
 * @implements {Type} Type implementation.
 * @template State Generics paramter.
 */
export declare class Impl<State> implements Type<State> {
    private store;
    constructor(initialState: State, scheduler: Scheduler, ...reducers: Observable<RxReducer<State, any>>[]);
    readonly stateInfoStream: Observable<StateInfo.Type<State, any>>;
    readonly stateStream: Observable<State>;
}
